@@ -91,7 +91,8 @@ class GrlTrackerPlaylists(GObject.GObject):
             "NeverPlayed": NeverPlayed(**args),
             "RecentlyPlayed": RecentlyPlayed(**args),
             "RecentlyAdded": RecentlyAdded(**args),
-            "Favorites": Favorites(**args)
+            "Favorites": Favorites(**args),
+            "InsufficientTagged": InsufficientTagged(**args),
         }
 
         for playlist in smart_playlists.values():
@@ -327,7 +328,6 @@ class Playlist(GObject.GObject):
         self._model = None
         self._source = source
         self._coremodel = application.props.coremodel
-        self._coreselection = application.props.coreselection
         self._log = application.props.log
         self._songs_hash = songs_hash
         self._tracker = tracker_wrapper.props.local_db
@@ -638,79 +638,24 @@ class Playlist(GObject.GObject):
 
         :param list coresongs: list of Coresong
         """
-        def _add_to_model(source, op_id, media, remaining, error):
-            if not media:
-                self.props.count = self._model.get_n_items()
-                return
-
-            coresong = CoreSong(self._application, media)
-            self._bind_to_main_song(coresong)
-            if coresong not in self._songs_todelete:
-                self._model.append(coresong)
-
-        def _requery_media(conn, res, coresong):
+        def _add_to_model(
+                connection: Tracker.SparqlConnection, result: Gio.AsyncResult,
+                coresong: CoreSong) -> None:
             if self._model is None:
                 return
 
-            media_id = coresong.props.media.get_id()
-            pl_id = self.props.pl_id
-            miner_fs_busname = self._tracker_wrapper.props.miner_fs_busname
-            query = """
-            SELECT
-                %(media_type)s AS ?type
-                ?song AS ?id
-                ?url
-                ?title
-                ?artist
-                ?album
-                ?duration
-                ?tag AS ?favorite
-                nie:contentAccessed(?song) AS ?lastPlayed
-                nie:usageCounter(?song) AS ?playCount
-            WHERE {
-                ?playlist a nmm:Playlist ;
-                          a nfo:MediaList ;
-                            nfo:hasMediaFileListEntry ?entry .
-                ?entry a nfo:MediaFileListEntry ;
-                         nfo:entryUrl ?url .
-                SERVICE <dbus:%(miner_fs_busname)s> {
-                    GRAPH tracker:Audio {
-                        SELECT
-                            ?song
-                            nie:title(?song) AS ?title
-                            nmm:artistName(nmm:artist(?song)) AS ?artist
-                            nie:title(nmm:musicAlbum(?song)) AS ?album
-                            nfo:duration(?song) AS ?duration
-                            ?url
-                        WHERE {
-                            ?song a nmm:MusicPiece ;
-                                  nie:isStoredAs ?url .
-                            %(location_filter)s
-                            FILTER (
-                                %(filter_clause_song)s
-                            )
-                        }
-                    }
-                }
-                OPTIONAL {
-                    ?song nao:hasTag ?tag .
-                    FILTER( ?tag = nao:predefined-tag-favorite )
-                }
-                FILTER (
-                    %(filter_clause_pl)s
-                )
-            } LIMIT 1
-            """.replace("\n", " ").strip() % {
-                "media_type": int(Grl.MediaType.AUDIO),
-                "filter_clause_song": f"?song = <{media_id}>",
-                "filter_clause_pl": f"?playlist = <{pl_id}>",
-                "location_filter": self._tracker_wrapper.location_filter(),
-                "miner_fs_busname": miner_fs_busname,
-            }
-
-            self._source.query(
-                query, self._METADATA_PLAYLIST_KEYS, self._fast_options,
-                _add_to_model)
+            try:
+                connection.update_finish(result)
+            except GLib.Error as error:
+                self._log.warning(
+                    "Unable to add to playlist {} song {}: {}".format(
+                        self.props.title, coresong.props.title, error.message))
+            else:
+                media = coresong.props.media
+                coresong_copy = CoreSong(self._application, media)
+                self._bind_to_main_song(coresong_copy)
+                self._model.append(coresong_copy)
+                self.props.count = self._model.get_n_items()
 
         for coresong in coresongs:
             query = """
@@ -738,7 +683,7 @@ class Playlist(GObject.GObject):
                 "song_uri": coresong.props.media.get_url()}
 
             self._tracker.update_blank_async(
-                query, None, _requery_media, coresong)
+                query, None, _add_to_model, coresong)
 
     def reorder(self, previous_position, new_position):
         """Changes the order of a songs in the playlist.
@@ -946,7 +891,7 @@ class NeverPlayed(SmartPlaylist):
         self.props.tag_text = "NEVER_PLAYED"
         # TRANSLATORS: this is a playlist name
         self._title = _("Never Played")
-        self.props.icon_name = "dialog-question-symbolic"
+        self.props.icon_name = "deaf-symbolic"
         self.props.query = """
         SELECT
             %(media_type)s AS ?type
@@ -1021,30 +966,46 @@ class RecentlyPlayed(SmartPlaylist):
             nie:usageCounter(?song) AS ?playCount
             ?tag AS ?favorite
         WHERE {
-            SERVICE <dbus:%(miner_fs_busname)s> {
-                GRAPH tracker:Audio {
-                    SELECT
-                        ?song
-                        nie:title(?song) AS ?title
-                        nie:isStoredAs(?song) AS ?url
-                        nmm:artistName(nmm:artist(?song)) AS ?artist
-                        nie:title(nmm:musicAlbum(?song)) AS ?album
-                        nfo:duration(?song) AS ?duration
-                        nmm:trackNumber(?song) AS ?trackNumber
-                        nmm:setNumber(nmm:musicAlbumDisc(?song))
-                            AS ?albumDiscNumber
-                    WHERE {
-                        ?song a nmm:MusicPiece .
-                        %(location_filter)s
+            {
+                SELECT
+                    ?song
+                    ?title
+                    ?url
+                    ?artist
+                    ?album
+                    ?duration
+                    ?trackNumber
+                    ?albumDiscNumber
+                    ?playCount
+                    ?tag
+                    ?lastPlayed
+                WHERE {
+                    SERVICE <dbus:%(miner_fs_busname)s> {
+                        GRAPH tracker:Audio {
+                            SELECT
+                                ?song
+                                nie:title(?song) AS ?title
+                                nie:isStoredAs(?song) AS ?url
+                                nmm:artistName(nmm:artist(?song)) AS ?artist
+                                nie:title(nmm:musicAlbum(?song)) AS ?album
+                                nfo:duration(?song) AS ?duration
+                                nmm:trackNumber(?song) AS ?trackNumber
+                                nmm:setNumber(nmm:musicAlbumDisc(?song))
+                                    AS ?albumDiscNumber
+                            WHERE {
+                                ?song a nmm:MusicPiece .
+                                %(location_filter)s
+                            }
+                        }
                     }
-                }
+                    ?song nie:contentAccessed ?lastPlayed ;
+                        nie:usageCounter ?playCount .
+                    OPTIONAL { ?song nao:hasTag ?tag .
+                               FILTER (?tag = nao:predefined-tag-favorite) }
+                } ORDER BY DESC(?lastPlayed) LIMIT 50
             }
-            ?song nie:contentAccessed ?lastPlayed .
-            FILTER ( ?lastPlayed > '%(compare_date)s'^^xsd:dateTime
-                     && EXISTS { ?song nie:usageCounter ?count .} )
-            OPTIONAL { ?song nao:hasTag ?tag .
-                       FILTER (?tag = nao:predefined-tag-favorite) }
-        } ORDER BY DESC(?lastPlayed) LIMIT 50
+            FILTER (?lastPlayed > '%(compare_date)s'^^xsd:dateTime)
+        }
         """.replace('\n', ' ').strip() % {
             "media_type": int(Grl.MediaType.AUDIO),
             'compare_date': compare_date,
@@ -1166,3 +1127,68 @@ class Favorites(SmartPlaylist):
             "location_filter": self._tracker_wrapper.location_filter(),
             "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
         }
+
+
+class InsufficientTagged(SmartPlaylist):
+    """Lacking tags to be displayed in the artist/album views"""
+
+    def __init__(self, **args):
+        super().__init__(**args)
+
+        self.props.tag_text = "INSUFFICIENT_TAGGED"
+        # TRANSLATORS: this is a playlist name indicating that the
+        # files are not tagged enough to be displayed in the albums
+        # or artists views.
+        self._title = _("Insufficiently Tagged")
+        self.props.icon_name = "question-round-symbolic"
+        self.props.query = """
+        SELECT
+            %(media_type)s AS ?type
+            ?song AS ?id
+            ?title
+            ?url
+            ?artist
+            ?album
+            ?duration
+            ?trackNumber
+            ?albumDiscNumber
+            nie:usageCounter(?song) AS ?playCount
+            ?tag AS ?favorite
+        WHERE {
+            SERVICE <dbus:%(miner_fs_busname)s> {
+                GRAPH tracker:Audio {
+                    SELECT
+                        ?song
+                        nie:title(?song) AS ?title
+                        nie:isStoredAs(?song) AS ?url
+                        nmm:artistName(nmm:artist(?song)) AS ?artist
+                        nie:title(nmm:musicAlbum(?song)) AS ?album
+                        nfo:duration(?song) AS ?duration
+                        nmm:trackNumber(?song) AS ?trackNumber
+                        nmm:setNumber(nmm:musicAlbumDisc(?song))
+                            AS ?albumDiscNumber
+                    WHERE {
+                        {
+                            ?song a nmm:MusicPiece .
+                            %(location_filter)s
+                            FILTER NOT EXISTS {
+                                ?song nmm:musicAlbum ?album
+                            }
+                        } UNION {
+                            ?song a nmm:MusicPiece .
+                            %(location_filter)s
+                            FILTER NOT EXISTS {
+                                ?song nmm:artist ?artist
+                            }
+                        }
+                    }
+                }
+            }
+            OPTIONAL { ?song nao:hasTag ?tag .
+                       FILTER (?tag = nao:predefined-tag-favorite) }
+        }
+        """.replace('\n', ' ').strip() % {
+            "media_type": int(Grl.MediaType.AUDIO),
+            "location_filter": self._tracker_wrapper.location_filter(),
+            "miner_fs_busname": self._tracker_wrapper.props.miner_fs_busname,
+        }
